class ClientsSharedConfig {
    public apiURL: string;

    public assignment: {
        filename: string, neededFiles: Array<string>, name: string, secret: string;
    };

    public gitlab: {
        dojoAccount: { id: number; username: string; };
    };

    public readonly dockerCompose: {
        projectName: string
    };

    public readonly exerciseResultsFolderMaxSizeInBytes: number;

    public readonly filenames: {
        results: string;
    };

    constructor() {
        this.apiURL = process.env.API_URL || '';

        this.assignment = {
            filename   : process.env.ASSIGNMENT_FILENAME || '',
            neededFiles: JSON.parse(process.env.EXERCISE_NEEDED_FILES || '[]'),
            name  : process.env.DOJO_ASSIGNMENT_NAME || '',
            secret: process.env.DOJO_ASSIGNMENT_SECRET || ''
        };

        this.gitlab = {
            dojoAccount: {
                id      : Number(process.env.GITLAB_DOJO_ACCOUNT_ID) || -1,
                username: process.env.GITLAB_DOJO_ACCOUNT_USERNAME || ''
            }
        };

        this.dockerCompose = {
            projectName: process.env.DOCKER_COMPOSE_PROJECT_NAME || ''
        };

        this.exerciseResultsFolderMaxSizeInBytes = Number(process.env.EXERCISE_RESULTS_FOLDER_MAX_SIZE_IN_BYTES || 0);

        this.filenames = {
            results: process.env.EXERCISE_RESULTS_FILENAME || ''
        };
    }
}


export default new ClientsSharedConfig();
